local HSTBlizzUI, C, L = unpack(select(2, ...))
local MODULE_NAME = "BlizzCompactRaidUI"

--[===[@debug@
print(MODULE_NAME)
--@end-debug@]===]

local PLUGIN = LibStub("WarlockHealthstoneTracker-1.0", 1)

---------------------------------------------
-- CONSTANTS
---------------------------------------------
local HEALTHSTONE_TEXTURE = [[Interface\ICONS\INV_Stone_04]]
local HEALTHSTONE_TEXTURE_SCALING = 0.1


---------------------------------------------
-- UTILITIES
---------------------------------------------
local function isUnitHealthstoneElligible(unit)
    return ( UnitExists(unit) and ( unit:match("^raid%d+$") or unit:match("^party%d+$") or unit == "player" ) )
end

local function createHealthstoneIcon(frame)
    if ( not frame.Healthstone ) then
        frame.Healthstone = CreateFrame("Frame", nil, frame)
        frame.Healthstone:SetHeight(11)
        frame.Healthstone:SetWidth(11)
        frame.Healthstone:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 1, -1)
        frame.Healthstone.texture = frame.Healthstone:CreateTexture(nil, "Artwork", nil)
        frame.Healthstone.texture:SetTexture(HEALTHSTONE_TEXTURE)
        frame.Healthstone.texture:SetAllPoints()
        frame.Healthstone.texture:SetTexCoord(0+HEALTHSTONE_TEXTURE_SCALING, 1-HEALTHSTONE_TEXTURE_SCALING, 0+HEALTHSTONE_TEXTURE_SCALING, 1-HEALTHSTONE_TEXTURE_SCALING)
        frame.Healthstone:Hide()
    end
end

local function findRaidFrame(unit)
    local guid = UnitGUID(unit)
    for _,type in ipairs({ "raid", "flagged" }) do
        local reservations = CompactRaidFrameContainer.frameReservations[type]
        local frame = CompactRaidFrameReservation_GetReservation(reservations, guid)
        if ( frame and frame.inUse ) then
            return frame
        end
    end

    return nil
end


---------------------------------------------
-- BLIZZARD RAID FRAMES
---------------------------------------------
local function updateFrameHealthstone(self)
    if ( not self ) then
        return
    end

    if ( C:is("ShowRaidHealthstones") and isUnitHealthstoneElligible(self.unit) ) then
        local hasHealthstone = PLUGIN:PlayerHasHealthstone(UnitName(self.unit))
        --@alpha@
        HSTBlizzUI:debug(hasHealthstone and "Show" or "Hide", "raid healthstone for", self.unit)
        --@end-alpha@

        createHealthstoneIcon(self)
        if ( hasHealthstone ) then
            self.Healthstone:SetSize(self.buffFrames[1]:GetSize())
        end
        self.Healthstone:SetShown(hasHealthstone)
    elseif ( self.Healthstone ) then
        self.Healthstone:Hide()
    end
end

local function handleHealthstoneUpdate(event, unitName, hasHealthstone)
    --[===[@debug@
    HSTBlizzUI:debug(unitName, "ShowRaidHealthstones", C:is("ShowRaidHealthstones"), "inRaid", UnitInRaid(unitName), "hasHealthstone", hasHealthstone)
    --@end-debug@]===]

    if ( C:is("ShowRaidHealthstones") ) then
        if ( GetCVar("useCompactPartyFrames") == "1" and UnitInParty(unitName) ) then  -- Configured to use raid frames in party
            for _,unit in ipairs(CompactRaidFrameContainer.partyUnits) do
                if ( UnitIsUnit(unit, unitName) ) then
                    local frame = findRaidFrame(unit)
                    if ( frame and frame.inUse ) then
                        updateFrameHealthstone(frame)
                    end
                    break
                end
            end


            -- RaidGroupFrames
            for i = 0, MEMBERS_PER_RAID_GROUP do
                local frame = _G["CompactPartyFrameMember"..i];
                if ( frame and UnitIsUnit(frame.unit, unitName) ) then
                    updateFrameHealthstone(frame)
                end
            end
        elseif ( UnitInRaid(unitName) ) then
            local raidId = UnitInRaid(unitName)
            local unit = "raid" .. raidId
            local frame = findRaidFrame(unit)
            if ( frame and frame.inUse ) then
                updateFrameHealthstone(frame)
            end

            -- RaidGroupFrames
            local _, _, subgroup = GetRaidRosterInfo(raidId);
            for i = 0, MEMBERS_PER_RAID_GROUP do
                local frame = _G["CompactRaidGroup" ..subgroup.."Member"..i];
                if ( frame and frame.unit == unit ) then
                    updateFrameHealthstone(frame)
                end
            end
        end


    end
end

local function handleOptionsUpdate(option, newValue)
    if ( option == "ShowRaidHealthstones" ) then
        for i = 1,100 do -- iterate through all frames until one doesn't exist
            local frame = _G["CompactRaidFrame"..i]
            if ( not frame or not frame.inUse ) then
                return
            end

            updateFrameHealthstone(frame)
        end
    end
end


---------------------------------------------
-- INITIALIZE
---------------------------------------------
HSTBlizzUI.RegisterCallback(MODULE_NAME, "initialize", function()
    --@alpha@
    HSTBlizzUI:debug("initalize module", MODULE_NAME)
    --@end-alpha@

    -- Update healthstone icons when updating raid buff icons
    hooksecurefunc("CompactUnitFrame_UpdateName", updateFrameHealthstone)

    -- Receive healthstone updates
    PLUGIN.RegisterCallback(HSTBlizzUI.ADDON_NAME .. "." .. MODULE_NAME, "updateUnitHealthstone", handleHealthstoneUpdate)

    -- Receive option changed updates
    C.RegisterListener(MODULE_NAME, "ShowRaidHealthstones", handleOptionsUpdate)
end)
