local HSTBlizzUI, C, L = unpack(select(2, ...))
local MODULE_NAME = "BlizzPulloutRaidUI"

--[===[@debug@
print(MODULE_NAME)
--@end-debug@]===]

local PLUGIN = LibStub("WarlockHealthstoneTracker-1.0", 1)


---------------------------------------------
-- CONSTANTS
---------------------------------------------
local HEALTHSTONE_TEXTURE = [[Interface\ICONS\INV_Stone_04]]
local HEALTHSTONE_TEXTURE_SCALING = 0.1


---------------------------------------------
-- UTILITIES
---------------------------------------------
local function isUnitHealthstoneElligible(unit)
    return ( UnitExists(unit) and ( unit:match("^raid%d+$") or unit:match("^party%d+$") or unit == "player" ) )
end

local function createHealthstoneIcon(frame)
    if ( not frame.Healthstone ) then
        frame.Healthstone = CreateFrame("Frame", nil, frame)
        frame.Healthstone:SetHeight(11)
        frame.Healthstone:SetWidth(11)
        frame.Healthstone:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 1, -1)
        frame.Healthstone.texture = frame.Healthstone:CreateTexture(nil, "Artwork", nil)
        frame.Healthstone.texture:SetTexture(HEALTHSTONE_TEXTURE)
        frame.Healthstone.texture:SetAllPoints()
        frame.Healthstone.texture:SetTexCoord(0+HEALTHSTONE_TEXTURE_SCALING, 1-HEALTHSTONE_TEXTURE_SCALING, 0+HEALTHSTONE_TEXTURE_SCALING, 1-HEALTHSTONE_TEXTURE_SCALING)
        frame.Healthstone:Hide()
    end
end

local function findRaidFrame(unit)
    local guid = UnitGUID(unit)
    for _,type in ipairs({ "raid", "flagged" }) do
        local reservations = CompactRaidFrameContainer.frameReservations[type]
        local frame = CompactRaidFrameReservation_GetReservation(reservations, guid)
        if ( frame and frame.inUse ) then
            return frame
        end
    end

    return nil
end


---------------------------------------------
-- BLIZZARD RAID FRAMES
---------------------------------------------
local function updateFrameHealthstone(self)
    if ( not self or not self:IsShown() ) then
        return
    end

    for i=1, self.numPulloutButtons do
        local button = self.buttons[i]
        if ( C:is("ShowRaidHealthstones") and isUnitHealthstoneElligible(button.unit) ) then
            local hasHealthstone = PLUGIN:PlayerHasHealthstone(UnitName(button.unit))
            --@alpha@
            HSTBlizzUI:debug(hasHealthstone and "Show" or "Hide", "raid healthstone for", self.unit)
            --@end-alpha@

            createHealthstoneIcon(button)
            button.Healthstone:SetShown(hasHealthstone)
        elseif ( button.Healthstone ) then
            button.Healthstone:Hide()
        end
    end
end

local function handleHealthstoneUpdate(event, unitName, hasHealthstone)
    --[===[@debug@
    HSTBlizzUI:debug(unitName, "ShowRaidHealthstones", C:is("ShowRaidHealthstones"), "inRaid", UnitInRaid(unitName), "hasHealthstone", hasHealthstone)
    --@end-debug@]===]

    if ( C:is("ShowRaidHealthstones") ) then
        for i = 1, NUM_RAID_PULLOUT_FRAMES do
            local self = _G["RaidPullout" .. i]
            for i=1, self.numPulloutButtons do
                local button = self.buttons[i]
                if ( isUnitHealthstoneElligible(button.unit) and UnitIsUnit(button.unit, unitName) ) then
                    createHealthstoneIcon(button)
                    button.Healthstone:SetShown(hasHealthstone)
                end
            end
        end
    end
end

local function handleOptionsUpdate(option, newValue)
    if ( option == "ShowRaidHealthstones" ) then
        for i = 1, NUM_RAID_PULLOUT_FRAMES do
            updateFrameHealthstone(_G["RaidPullout" .. i])
        end
    end
end


---------------------------------------------
-- INITIALIZE
---------------------------------------------
HSTBlizzUI.RegisterCallback(MODULE_NAME, "initialize", function()
    HSTBlizzUI:InitializeOnDemand("Blizzard_RaidUI", function()
        -- Update healthstone icons when updating raid buff icons
        hooksecurefunc("RaidPullout_Update", updateFrameHealthstone)

        -- Receive healthstone updates
        PLUGIN.RegisterCallback(HSTBlizzUI.ADDON_NAME .. "." .. MODULE_NAME, "updateUnitHealthstone", handleHealthstoneUpdate)

        -- Receive option changed updates
        C.RegisterListener(MODULE_NAME, "ShowRaidHealthstones", handleOptionsUpdate)
    end)
end)
